import json
import requests
import time
from hashlib import sha1
from werkzeug import urls

import logging
_logger = logging.getLogger(__name__)


class OvhTooManyLines(Exception):
    # raise if number of lines in invoice < max_line (see get_details)
    pass


class OvhAPI():
    def __init__(self, HOST, AK, AS, CK, v='1.0'):
        self.AK = AK
        self.AS = AS
        self.CK = CK
        self.HOST = f'{HOST}/{v}'
        resp = requests.get(self.HOST + "/auth/time")
        self.time_delta = int(resp.content.decode()) - int(time.time())

    def _sign(self, METHOD, QUERY, BODY, TS):
        tosign = '+'.join([self.AS, self.CK, METHOD, QUERY, BODY, TS])
        return f'$1${sha1(tosign.encode()).hexdigest()}'

    def _do(self, method, query, body=''):
        # preprocess
        if query.startswith('/'):
            query = self.HOST + query

        if method == 'GET' and body:
            query = query + '?' + urls.url_encode(body)
            body = ''

        if body:
            body = json.dumps(body, separators=(",", ":"))

        # compute sign
        ts = str(int(time.time()) + self.time_delta)
        headers = {
            'X-Ovh-Application': self.AK,
            'X-Ovh-Consumer': self.CK,
            'X-Ovh-Timestamp': ts,
            'X-Ovh-Signature': self._sign(method, query, body, ts),
            'Content-type': "application/json",
        }

        # do request
        _logger.debug(query)
        resp = requests.request(method, query, data=body, headers=headers)
        if resp.status_code != 200:
            _logger.warning(resp.content)
        _logger.debug(resp.content)
        resp.raise_for_status()
        return resp

    def getBills(self, date_from=None, date_to=None):
        body = {
            'date.from': date_from,
            'date.to': date_to
        }
        invoices = self._do('GET', '/me/bill', body).json()
        return invoices

    def getBill(self, billId):
        # {
        #     'billId': 'FR56709999',
        #      'category': 'autorenew',
        #      'date': '2023-05-01T07:54:57+02:00',
        #      'orderId': 189879274,
        #      'password': 'xxx',
        #      'pdfUrl': 'https://www.ovh.com/cgi-bin/order/bill.pdf?reference=FR56709999&timestamp=1688633934&esign=xxx',
        #      'priceWithTax': {'currencyCode': 'EUR', 'text': '54.90 €', 'value': 54.9},
        #      'priceWithoutTax': {'currencyCode': 'EUR', 'text': '54.90 €', 'value': 54.9},
        #      'tax': {'currencyCode': 'EUR', 'text': '0.00 €', 'value': 0},
        #      'url': 'https://www.ovh.com/cgi-bin/order/bill.cgi?reference=FR56709999&timestamp=1688633934&esign=xxx'
        # }
        query = f'/me/bill/{billId}'
        invoice = self._do('GET', query).json()
        return invoice

    def getBillDetails(self, billId, ignoreZero=True, max_lines=200):
        # [{
        #      'billDetailId': 'FR2747688888',
        #      'description': 'Snapshots Public Cloud - ',
        #      'domain': 'fee1ac928c6a4905b2cdeb73cc2b9552',
        #      'periodEnd': None,
        #      'periodStart': None,
        #      'quantity': '1',
        #      'unitPrice': {'currencyCode': 'EUR', 'text': '2.23 €', 'value': 2.23}
        #      'totalPrice': {'currencyCode': 'EUR', 'text': '2.23 €', 'value': 2.23},
        # }, ...]
        return self.getDetails('bill', billId, ignoreZero=ignoreZero, max_lines=max_lines)

    def getRefunds(self, date_from=None, date_to=None, order_id=None):
        body = {
            'orderId': order_id,
            'date.from': date_from,
            'date.to': date_to
        }
        refunds = self._do('GET', '/me/refund', body).json()
        return refunds

    def getRefund(self, refundId):
        # {
        #     'date': '2021-05-12T14:15:48+02:00',
        #     'orderId': 148584439,
        #     'originalBillId': 'FR40110737',
        #     'password': 'BbdtQOoeBP',
        #     'pdfUrl': 'https://www.ovh.com/cgi-bin/order/bill.pdf?reference=AFR751951&timestamp=1688634807&esign=xxx',
        #     'priceWithTax': {'currencyCode': 'EUR',
        #                      'text': '-1778.37 €',
        #                      'value': -1778.37},
        #     'priceWithoutTax': {'currencyCode': 'EUR',
        #                         'text': '-1778.37 €',
        #                         'value': -1778.37},
        #     'refundId': 'AFR751951',
        #     'tax': {'currencyCode': 'EUR', 'text': '0.00 €', 'value': 0},
        #     'url': 'https://www.ovh.com/cgi-bin/order/bill.cgi?reference=AFR751951&timestamp=1688634807&esign=xxx'
        # }
        query = f'/me/refund/{refundId}'
        refund = self._do('GET', query).json()
        return refund

    def getRefundDetails(self, refundId, ignoreZero=True, max_lines=200):
        # [{
        #     'refundDetailId': 'AFR3309863',
        #     'refundId': 'AFR899562'
        #     'domain': '*',
        #     'quantity': '1',
        #     'unitPrice': { 'value': -4761.13, 'text': '-4761.13 €', 'currencyCode': 'EUR'},
        #     'totalPrice': {'currencyCode': 'EUR', 'text': '-4761.13 €', 'value': -4761.13},
        #     'description': 'Geste commercial - Professional Services - Infrastructure - 1 mois',
        #     'reference': 'giftAuto.onsite.infra.a.b',
        # }, ...]
        return self.getDetails('refund', refundId, ignoreZero=ignoreZero, max_lines=max_lines)

    def getDetails(self, doc_type, doc_id, ignoreZero=True, max_lines=200):
        # See getInvoiceDetails & getRefundDetails for response format
        query = f'/me/{doc_type}/{doc_id}/details'
        detail_ids = self._do('GET', query).json()

        if max_lines and len(detail_ids) > max_lines:
            raise OvhTooManyLines()

        for _id in detail_ids:
            detail = self._do('GET', query + f'/{_id}').json()
            if detail['unitPrice']['value'] or not ignoreZero:
                yield detail
